package cmdbuild.elements;

import cmdbuild.db.DBService;
import cmdbuild.collection.AssetBean;
import cmdbuild.collection.AttributeBean;
import cmdbuild.collection.ClassBean;
import cmdbuild.collection.HistoryBean;
import cmdbuild.collection.IdBean;
import cmdbuild.collection.InheritsBean;
import cmdbuild.collection.LabelsBean;
import cmdbuild.presentation.HTMLclass;
import cmdbuild.presentation.Presentation;

import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Types;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.LinkedList;
import java.util.List;
import java.util.Vector;

public class CMDBclass {

	String CREATE_CLASS = "{ ?= call createClass(?,?,?,?,?,?) }"; // nome,descrizione,stato,parentclasse,superclass?,manager
	String MODIFY_CLASS = "{ ?= call modifyClass(?,?,?,?,?,?) }";  // nome,nuovonome,descrizione,stato,superclass?,manager
	String DELETE_CLASS = "{ ?= call deleteClass(?) }";  // nome
	String CREATE_ATTRIBUTE = "{ ?= call createAttribute(?,?,?,?,?,?,?,?,?,?) }"; // nomeclasse,nome,indice,descrizione,tipo,lookup,obbligatorio,default,baseDSP,stato
	String MODIFY_ATTRIBUTE = "{ ?= call modifyAttribute(?,?,?,?,?,?,?,?,?,?,?) }"; // nomeclasse,nome,nuovonome,indice,descrizione,tipo,lookup,obbligatorio,default,baseDSP,stato
	
	protected DBService db;
	
	protected ClassBean bean; 
	protected Hashtable attributeHash;
	protected Vector attributeVector;
	protected Presentation presentation;
	protected String classManager;
	protected String user; // this is the logged user

	public CMDBclass() {
		super();
		// TODO Auto-generated constructor stub
	}
	
	public CMDBclass(DBService db) {
		super();
		// TODO Auto-generated constructor stub
		this.db = db;
		// patch that prevents no data and classes in the database
		this.bean = new ClassBean();
		this.getBean().setManager("");
	}

	// usually used as external call
	public CMDBclass load(CMDBclass obj, String manager){
		if (obj.getClass().getName().indexOf("HTML")==-1){ // is CMDBxxxxx
			//System.out.println("Loader: found CMDB");
			String path = "cmdbuild.elements.CMDB";
			try {
				// save old values
				DBService dbs = obj.db;
				ClassBean cb = obj.bean; 
				Hashtable aHash = obj.attributeHash;
				Vector aVector = obj.attributeVector;
				Presentation p = obj.presentation;
				//String cManager = obj.classManager;
				String us = obj.user;
				obj = (CMDBclass)(Class.forName(path+manager).newInstance());
				// reload old values
				obj.db = dbs;
				obj.bean = cb;
				obj.attributeHash = aHash;
				obj.attributeVector = aVector;
				obj.presentation = p;
				obj.classManager = manager; // now has a new manager
				obj.user = us;
				System.out.println("loading "+path+manager);
			} catch (InstantiationException e) {
				// TODO Auto-generated catch block
				System.out.println("InstantiationException");
				//e.printStackTrace();
			} catch (IllegalAccessException e) {
				// TODO Auto-generated catch block
				System.out.println("IllegalAccessException");
				//e.printStackTrace();
			} catch (ClassNotFoundException e) {
				// TODO Auto-generated catch block
		       System.out.println("ClassNotFoundException");
				//e.printStackTrace();
			}
			return obj;
		} else {
			return null;
		}	
	} 
	// usually used by getPresentation 
	public Presentation load(Presentation obj, String manager){
		if (obj.getClass().getName().indexOf("HTML")!=-1){ // is HTMLxxxxx
			String path = "cmdbuild.presentation.HTML";
			//System.out.println("Loader: found HTML");
			try {
				obj = (Presentation)(Class.forName(path+manager).newInstance());
				System.out.println("loading "+path+manager);
			} catch (InstantiationException e) {
				// TODO Auto-generated catch block
				System.out.println("InstantiationException");
				//e.printStackTrace();
			} catch (IllegalAccessException e) {
				// TODO Auto-generated catch block
				System.out.println("IllegalAccessException");
				//e.printStackTrace();
			} catch (ClassNotFoundException e) {
				// TODO Auto-generated catch block
		       System.out.println("ClassNotFoundException");
				//e.printStackTrace();
			}
			return obj;
		} else {
			return null;
		}	
	} 
	
	public boolean isEmpy(String name) throws SQLException{
		boolean empty = false;
		String query = "SELECT isempty('"+name+"');";
		ResultSet rs = null;
		PreparedStatement stm = null;
		System.out.println("Query ISEMPTY="+query);
		Connection con = db.getConnection();
		try {
			if ( con != null ) {
				stm = con.prepareStatement(query);
				rs = stm.executeQuery();
				if ( rs.next() ) { // while ( rs.next() ) {
					empty = rs.getBoolean(1);
				}
			}
		} catch( SQLException ex ) {
			throw ex;
		} finally { // added**
			DBService.close(rs, stm, con);
		}
		return empty;
	}
	// *********
	// **class**
	// *********
	public boolean createClass(ClassBean bean) throws SQLException {
		boolean ok = false;
		CallableStatement stm = null;
		System.out.println("createclass("+bean.getName()+","+bean.getDescription()+","+bean.getStatus()+","+bean.getParent()+","+bean.isSuperClass()+","+bean.getManager()+")");
		Connection con = db.getConnection();
		try {
			stm = con.prepareCall(CREATE_CLASS);
			stm.registerOutParameter(1,Types.BOOLEAN);
			stm.setString(2,bean.getName());
			stm.setString(3,bean.getDescription());
			stm.setString(4,bean.getStatus());
			stm.setString(5,bean.getParent());
			stm.setBoolean(6,bean.isSuperClass());
			stm.setString(7,bean.getManager());
			stm.execute();
			ok = true;
		} catch (SQLException ex) {
			throw ex;
		} finally {
			DBService.close(null, stm, con);
		}
		return ok;
	}
	public boolean modifyClass(String newname, ClassBean bean) throws SQLException {
		boolean ok = false;
		CallableStatement stm = null;
		System.out.println("modifyclass("+bean.getName()+","+newname+","+bean.getDescription()+","+bean.getStatus()+","+bean.isSuperClass()+","+bean.getManager()+")");
		Connection con = db.getConnection();
		try {
			stm = con.prepareCall(MODIFY_CLASS);
			stm.registerOutParameter(1,Types.BOOLEAN);
			stm.setString(2,bean.getName());
			stm.setString(3,newname);
			stm.setString(4,bean.getDescription());
			stm.setString(5,bean.getStatus());
			stm.setBoolean(6,bean.isSuperClass());
			stm.setString(7,bean.getManager());
			stm.execute();
			ok = true;
		} catch (SQLException ex) {
			throw ex;
		} finally {
			DBService.close(null, stm, con);
		}
		return ok;
	}
	public boolean modifyClassComment(String tabname, String comment) throws SQLException {
		boolean ok = false;
		Statement stm = null;
		ResultSet rs = null;
		String query="";
		query = "COMMENT ON TABLE \""+tabname+"\" IS '"+comment+"';";			
		System.out.println("Query MODIFYCLASSCOMMENT="+query);
		Connection con = db.getConnection();
		try {
			if ( con != null ) {
				stm = con.createStatement();
				stm.executeUpdate(query);
				ok = true;
			}
		} catch( SQLException ex ) {
			throw ex;
		} finally {
			DBService.close(rs, stm, con);
		}
		return ok;
	}
	public boolean deleteClass(String name) throws SQLException{
		boolean ok = false;
		CallableStatement stm = null;
		System.out.println("deleteclass("+name+")");
		Connection con = db.getConnection();
		try {
			stm = con.prepareCall(DELETE_CLASS);
			stm.registerOutParameter(1,Types.BOOLEAN);
			stm.setString(2,name);
			stm.execute();
			ok = true;
		} catch (SQLException ex) {
			throw ex;
		} finally {
			DBService.close(null, stm, con);
		}
		return ok;
	}
	// ** SELECT QUERIES**
	public ClassBean selectClassByName(String name) throws SQLException {
		ResultSet rs = null;
		PreparedStatement stm = null;
		ClassBean obj = null; 
		String query = "";
		query = "SELECT DISTINCT classid,classname,classcomment,classidcategory,classcategory,classdescription,classmode,issuperclass,classmanager,classstatus FROM cmdbclasscatalog WHERE classname='"+name+"' ;";
		System.out.println("Query SELECTCLASSBYNAME="+query);
		Connection con = db.getConnection();
		try {
			if ( con != null ) { // 
				stm = con.prepareStatement(query);
				//stm.setString(1,name);
				rs = stm.executeQuery();
				if ( rs.next() ) {
					obj = setClassObject(rs);
					this.bean = obj;
				}
			}
		} catch( SQLException ex ) {
			throw ex;
		} finally {
			DBService.close(rs, stm, con);
		}
		return obj;
	}
	public ClassBean selectClassById(int classId) throws SQLException {
		ClassBean obj = new ClassBean();
		String query = "SELECT DISTINCT classid,classname,classcomment,classidcategory,classcategory,classdescription,classmode,issuperclass,classmanager,classstatus FROM cmdbclasscatalog WHERE classid="+classId+" ;";
		ResultSet rs = null;
		PreparedStatement stm = null;
		System.out.println("Query SELECTCLASSBYID="+query);
		Connection con = db.getConnection();
		try {
			if ( con != null ) {
				stm = con.prepareStatement(query);
				rs = stm.executeQuery();
				if ( rs.next() ) { // while ( rs.next() ) {
					obj = setClassObject(rs);
					this.bean = obj;
				}
			}
		} catch( SQLException ex ) {
			throw ex;
		} finally { // added**
			DBService.close(rs, stm, con);
		}
		return obj;
	}
	public List selectAllClass() throws SQLException {
		List list = new LinkedList();
		ResultSet rs = null;
		PreparedStatement stm = null; // eventualmente modificare il nome
		String query = "SELECT DISTINCT classid,classname,classcomment,classidcategory,classcategory,classdescription,classmode,issuperclass,classmanager,classstatus FROM cmdbclasscatalog WHERE classmode<>'reserved';";
		System.out.println("Query SELECTALLCLASS="+query);
		Connection con = db.getConnection();
		try {
			if ( con != null ) {
				stm = con.prepareStatement(query);
				rs = stm.executeQuery();
				ClassBean obj = null;
				while( rs.next() ) {
					obj = setClassObject(rs);
					list.add(obj);
				}
			}
		} catch( SQLException ex ) {
			throw ex;
		} finally {
			DBService.close(rs, stm, con);
		}
		return list;
	}
	public List selectSuperClasses() throws SQLException {
		List list = new LinkedList();
		ResultSet rs = null;
		PreparedStatement stm = null; 
		String query = "SELECT DISTINCT classid,classname,classcomment,classidcategory,classcategory,classdescription,classmode,issuperclass,classmanager,classstatus FROM cmdbclasscatalog WHERE issuperclass='true'";
		System.out.println("Query SELECTSUPERCLASSES="+query);
		Connection con = db.getConnection();
		try {
			if ( con != null ) {
				stm = con.prepareStatement(query);
				rs = stm.executeQuery();
				ClassBean obj = null;
				while( rs.next() ) {
					obj = setClassObject(rs);
					list.add(obj);
				}
			}
		} catch( SQLException ex ) {
			throw ex;
		} finally {
			DBService.close(rs, stm, con);
		}
		return list;
	}

	// *************
	// **attribute**
	// *************
	public boolean createAttribute(String tabname, AttributeBean bean) throws SQLException {
		boolean ok = false;
		CallableStatement stm = null;
		System.out.println("createattribute("+tabname+","+bean.getName()+","+bean.getIndex()+","+bean.getDescription()+","+bean.getSQLType()+","+bean.getLookup()+","+bean.getMandatory()+","+bean.getDefaultValue()+","+bean.getBaseDSP()+","+bean.getStatus()+")");
		Connection con = db.getConnection();
		try {
			stm = con.prepareCall(CREATE_ATTRIBUTE);
			stm.registerOutParameter(1,Types.BOOLEAN);
			stm.setString(2,tabname);
			stm.setString(3,bean.getName());
			stm.setInt(4,bean.getIndex());
			stm.setString(5,bean.getDescription());
			stm.setString(6,bean.getSQLType());
			stm.setString(7,bean.getLookup());
			stm.setString(8,bean.getMandatory());
			stm.setString(9,bean.getDefaultValue());
			stm.setString(10,bean.getBaseDSP());
			stm.setString(11,bean.getStatus());
			stm.execute();
			ok = true;
		} catch (SQLException ex) {
			throw ex;
		} finally {
			DBService.close(null, stm, con);
		}
		return ok;
	}
	public boolean modifyAttribute(String tabname, String newattrname, AttributeBean bean) throws SQLException {
		boolean ok = false;
		CallableStatement stm = null;
		System.out.println("modifyattribute("+tabname+","+bean.getName()+","+newattrname+","+bean.getIndex()+","+bean.getDescription()+","+bean.getSQLType()+","+bean.getLookup()+","+bean.getMandatory()+","+bean.getDefaultValue()+","+bean.getBaseDSP()+","+bean.getStatus()+")");
		Connection con = db.getConnection();
		try {
			stm = con.prepareCall(MODIFY_ATTRIBUTE);
			stm.registerOutParameter(1,Types.BOOLEAN);
			stm.setString(2,tabname);
			stm.setString(3,bean.getName());
			stm.setString(4,newattrname);
			stm.setInt(5,bean.getIndex());
			stm.setString(6,bean.getDescription());
			stm.setString(7,bean.getSQLType());
			stm.setString(8,bean.getLookup());
			stm.setString(9,bean.getMandatory());
			stm.setString(10,bean.getDefaultValue());
			stm.setString(11,bean.getBaseDSP());
			stm.setString(12,bean.getStatus());
			stm.execute();
			ok = true;
		} catch (SQLException ex) {
			throw ex;
		} finally {
			DBService.close(null, stm, con);
		}
		return ok;
	}
	public boolean modifyAttributeComment(String tabname, String aname, String comment) throws SQLException {
		boolean ok = false;
		Statement stm = null;
		ResultSet rs = null;
		String query="";
		query = "COMMENT ON COLUMN \""+tabname+"\".\""+aname+"\" IS '"+comment+"';";			
		System.out.println("Query MODIFYATTRIBUTECOMMENT="+query);
		Connection con = db.getConnection();
		try {
			if ( con != null ) {
				stm = con.createStatement();
				stm.executeUpdate(query);
				ok = true;
			}
		} catch( SQLException ex ) {
			throw ex;
		} finally {
			DBService.close(rs, stm, con);
		}
		return ok;
	}
	public boolean deleteAttribute(String className, String aName) throws SQLException{
		boolean ok = false;
		String query = "SELECT deleteattribute('"+className+"','"+aName+"');";
		ResultSet rs = null;
		PreparedStatement stm = null;
		System.out.println("Query DELETEATTRIBUTE="+query);
		Connection con = db.getConnection();
		try {
			if ( con != null ) {
				stm = con.prepareStatement(query);
				rs = stm.executeQuery();
				if ( rs.next() ) { // while ( rs.next() ) {
					ok = rs.getBoolean(1);
				}
			}
		} catch( SQLException ex ) {
			throw ex;
		} finally { // added**
			DBService.close(rs, stm, con);
		}
		return ok;
	}
	// **FOLLOW SELECT QUERIES**
	public  Vector selectAllAttribute(String name, String mode) throws SQLException {
		Vector attributeBeanVector = new Vector();
		ResultSet rs = null;
		PreparedStatement stm = null;
		AttributeBean obj = null;
		String query="";
		if (mode.equals("all")){
			query = "SELECT classid,classname,classcomment,classidcategory,classcategory,classdescription,classmode,issuperclass,classmanager,classstatus,attributename,attributetype,attributelength,attributenull,attributedefault,attributemode,attributedescription,attributelookup,attributebasedsp,attributestatus,attributeindex,attributedbindex,attributecomment FROM cmdbclasscatalog WHERE classname='"+name+"' ORDER BY attributeindex,attributedbindex;";	
		} else if (mode.equals("rw")){ // read and write
			query = "SELECT classid,classname,classcomment,classidcategory,classcategory,classdescription,classmode,issuperclass,classmanager,classstatus,attributename,attributetype,attributelength,attributenull,attributedefault,attributemode,attributedescription,attributelookup,attributebasedsp,attributestatus,attributeindex,attributedbindex,attributecomment FROM cmdbclasscatalog WHERE classname='"+name+"' AND (\"attributemode\"='read' OR \"attributemode\"='write') ORDER BY attributeindex,attributedbindex;";
		} else if (mode.equals("w")){
			query = "SELECT classid,classname,classcomment,classidcategory,classcategory,classdescription,classmode,issuperclass,classmanager,classstatus,attributename,attributetype,attributelength,attributenull,attributedefault,attributemode,attributedescription,attributelookup,attributebasedsp,attributestatus,attributeindex,attributedbindex,attributecomment FROM cmdbclasscatalog WHERE classname='"+name+"' AND \"attributemode\"='write' ORDER BY attributeindex,attributedbindex;";
		} 
		System.out.println("Query SELECTALLATTRIBUTE="+query);
		Connection con = db.getConnection();
		try {
			if ( con != null ) {
				stm = con.prepareStatement(query);
				rs = stm.executeQuery();
				while ( rs.next() ) {
					obj = setAttributeObject(rs);
					attributeBeanVector.add(obj);
				}
				if (attributeBeanVector.size()>0){
					this.bean = ( (AttributeBean)attributeBeanVector.elementAt(0) ).getClassBean();	
				}
			}	
		} catch( SQLException ex ) {
			throw ex;
		} finally {
			DBService.close(rs, stm, con);
		}
		return attributeBeanVector;
	}
	// used by modifyAttributeForm
	// there is no distinction between normal and special class because special class isn't editable
	public  AttributeBean selectAttributeByName(String tabname, String name) throws SQLException {
		ResultSet rs = null;
		PreparedStatement stm = null;
		AttributeBean obj = null;
		String query = "SELECT classid,classname,classcomment,classidcategory,classcategory,classdescription,classmode,issuperclass,classmanager,classstatus,attributename,attributetype,attributelength,attributenull,attributedefault,attributemode,attributedescription,attributelookup,attributebasedsp,attributestatus,attributeindex,attributedbindex,attributecomment FROM cmdbclasscatalog WHERE classname='"+tabname+"' AND attributename='"+name+"' ORDER BY attributeindex,attributedbindex";
		//String query = "select classid,classname,classidcategory,classdescription,attributename,attributetype,attributelength,attributenull,attributedefault,attributemode,attributedescription,attributelookup,attributebasedsp,attributestatus,attributeindex,attributedbindex from cmdbclasscatalog where classname='"+tabname+"' AND attributename='"+name+"' order by attributeindex,attributedbindex;";
		System.out.println("Query SELECTATTRIBUTEBYNAME="+query);
		Connection con = db.getConnection();
		try {
			if ( con != null ) {
				stm = con.prepareStatement(query);
				rs = stm.executeQuery();
				if ( rs.next() ) {
					obj = setAttributeObject(rs);
				}
			}
		} catch( SQLException ex ) {
			throw ex;
		} finally {
			DBService.close(rs, stm, con);
		}
		return obj;
	}
	// ********
	// **card**
	// ********
	public  boolean createCard(String className, Vector columnVector, String user) throws SQLException {
		// creating query
		int r=0;
		boolean result;
		Statement stm = null;
		ResultSet rs = null;
		String query = "INSERT INTO \""+className+"\"";
		String data = "";
		String value = "";
		Enumeration en = columnVector.elements();
		AssetBean ab = null;
		while (en.hasMoreElements()){
			ab = (AssetBean)en.nextElement();
			//System.out.println("nome="+ab.getAName()+" tipo="+ab.getAType()+" valore="+ab.getValue());
			if (data.equals("")){
				data = " (\""+ab.getName()+"\"";
			} else {
				data+= ",\""+ab.getName()+"\"";
			}
			if (value.equals("")){
				if (ab.getType().equals("numeric") ||ab.getType().equals("float8") || ab.getType().equals("int4") || ab.getType().equals("lookup")){ // without '' 
					value = " VALUES ("+ab.getValue()+"";
				} else { // with '' 
					value = " VALUES ('"+ab.getValue()+"'";	
				}
			} else {
				if (ab.getType().equals("numeric") ||ab.getType().equals("float8") || ab.getType().equals("int4") || ab.getType().equals("lookup")){ // without '' 
					value+= ","+ab.getValue()+"";
				} else { // with ''
					value+= ",'"+ab.getValue()+"'";					
				}
			}
		}
		if (user!=null){
			data=data+",\"User\")";
			value=value+",'"+user+"')";
		} else {
			data=data+")";
			value=value+")";
		}
		query+=data+value;
		System.out.println("Query CREATECARD="+query);
		Connection con = db.getConnection();
		try {
			if ( con != null ) {
				stm = con.createStatement();
				r= stm.executeUpdate(query);
			}
		} catch( SQLException ex ) {
			throw ex;
		} finally {
			DBService.close(rs, stm, con);
		}
		if (r==1) {
			result = true;
		} else {
			result = false;
		}
		return result;
	}
	public  boolean createCardBatch(String className, Vector tableVector, String user) throws SQLException {
		// creating query
		boolean result = false;
		Statement stm = null;
		Connection con = db.getConnection();
		if ( con != null ) {
			stm = con.createStatement();
		}
		ResultSet rs = null;
		String query = "";
		String data = "";
		String value = "";
		Enumeration en = tableVector.elements();
		Vector columnVector = null;
		while (en.hasMoreElements()){
			query = "INSERT INTO \""+className+"\"";
			data = "";
			value = "";
			columnVector = (Vector)en.nextElement();
			Enumeration enu = columnVector.elements();
			AssetBean ab = null;
			while (enu.hasMoreElements()){
				ab = (AssetBean)enu.nextElement();
				if (!ab.isToDelete()){
					//System.out.println("nome="+ab.getAName()+" tipo="+ab.getAType()+" valore="+ab.getValue());
					if (data.equals("")){
						data = " (\""+ab.getName()+"\"";
					} else {
						data+= ",\""+ab.getName()+"\"";
					}
					if (value.equals("")){
						if (ab.getType().equals("numeric") ||ab.getType().equals("float8") || ab.getType().equals("int4") || ab.getType().equals("lookup")){ // without '' 
							value = " VALUES ("+ab.getValue()+"";
						} else { // with '' 
							value = " VALUES ('"+ab.getValue()+"'";	
						}
					} else {
						if (ab.getType().equals("numeric") ||ab.getType().equals("float8") || ab.getType().equals("int4") || ab.getType().equals("lookup")){ // without '' 
							value+= ","+ab.getValue()+"";
						} else { // with ''
							value+= ",'"+ab.getValue()+"'";					
						}
					}
				}
			}
			if (user!=null){
				data=data+",\"User\")";
				value=value+",'"+user+"')";
			} else {
				data=data+")";
				value=value+")";
			}
			query+=data+value;
			System.out.println("Query CREATECARDBATCH="+query);
			stm.addBatch(query);
		}
		try {
			if ( con != null ) {
				stm.executeBatch();
				result = true;
			}
		} catch( SQLException ex ) {
			result = false;
			throw ex;
		} finally {
			DBService.close(rs, stm, con);
		}
		return result;
	}
	public  boolean modifyCard(String className, int Id, Vector columnVector, String user) throws SQLException {
		int r=0;
		boolean result;
		Statement stm = null;
		ResultSet rs = null;
		String query = "UPDATE \""+className+"\"";
		String data = "";
		String value = "";
		if (user!=null){ // adding user
			AssetBean us = new AssetBean();
			us.setName("User");
			us.setType("varchar");
			us.setSQLType("varchar");
			us.setValue(user);
			columnVector.add(us);
		}
		// creating query
		Enumeration en = columnVector.elements();
		AssetBean ab = null;
		while (en.hasMoreElements()){
			ab = (AssetBean)en.nextElement();
			if (data.equals("")){
				if (ab.getType().equals("numeric") ||ab.getType().equals("float8") || ab.getType().equals("int4") || ab.getType().equals("lookup")){ // without '' 
					data = " SET \""+ab.getName()+"\"="+ab.getValue()+"";
				} else { // with '' 
					if (ab.getValue()!=null){
						data = " SET \""+ab.getName()+"\"='"+ab.getValue()+"'";	
					} else { //value=null is without ''
						data = " SET \""+ab.getName()+"\"="+ab.getValue()+""; 
					}
				}
			} else {
				if (ab.getType().equals("numeric") ||ab.getType().equals("float8") || ab.getType().equals("int4") || ab.getType().equals("lookup")){ // without '' 
					data+= ",\""+ab.getName()+"\"="+ab.getValue()+"";
				} else { // with ''
					if (ab.getValue()!=null){
						data+= ",\""+ab.getName()+"\"='"+ab.getValue()+"'";
					} else { //value=null is without ''
						data+= ",\""+ab.getName()+"\"="+ab.getValue()+"";
					}
				}
			}
		}
		value=" WHERE \""+className+"\".\"Id\"="+Id;
		query+=data+value;
		System.out.println("Query MODIFYCARD="+query);
		Connection con = db.getConnection();
		try {
			if ( con != null ) {
				stm = con.createStatement();
				r= stm.executeUpdate(query) ;
			}
		} catch( SQLException ex ) {
			throw ex;
		} finally {
			DBService.close(rs, stm, con);
		}
		if (r==1) {
			result = true;
		} else {
			result = false;
		}
		return result;
	}
	public void setCardStatus(String className, int Id, String status, String user) throws SQLException {
		Statement stm = null;
		ResultSet rs = null;
		String query="";
		if (user!=null){
			query = "UPDATE \""+className+"\" SET \"Status\"='"+status+"',\"User\"='"+user+"' WHERE \"Id\"='"+Id+"'";			
		} else {
			query = "UPDATE \""+className+"\" SET \"Status\"='"+status+"' WHERE \"Id\"='"+Id+"'";
		}
		System.out.println("Query SETCARDSTATUS="+query);
		Connection con = db.getConnection();
		try {
			if ( con != null ) {
				stm = con.createStatement();
				stm.executeUpdate(query) ;
			}
		} catch( SQLException ex ) {
			throw ex;
		} finally {
			DBService.close(rs, stm, con);
		}
	}
	public String getCardStatus(String className, int Id) throws SQLException {
		String status="";
		PreparedStatement stm = null;
		ResultSet rs = null;
		String query = "SELECT \"Status\" FROM \""+className+"\" WHERE \"Id\"='"+Id+"'";
		System.out.println("Query GETCARDSTATUS="+query);
		Connection con = db.getConnection();
		try {
			if ( con != null ) {
				stm = con.prepareStatement(query);
				rs = stm.executeQuery();
				if ( rs.next() ) {
					status = rs.getString(1);
				}
			}
		} catch( SQLException ex ) {
			throw ex;
		} finally {
			DBService.close(rs, stm, con);
		}
		return status;
	}
	// overload
	public boolean setCardNotes(String className, int Id, String notes, String user) throws SQLException {
		Statement stm = null;
		ResultSet rs = null;
		String query="";
		boolean result = false;
		if (user!=null){
			query = "UPDATE \""+className+"\" SET \"Notes\"='"+notes+"',\"User\"='"+user+"' WHERE \"Id\"="+Id;			
		} else {
			query = "UPDATE \""+className+"\" SET \"Notes\"='"+notes+"' WHERE \"Id\"="+Id;
		}
		System.out.println("Query SETCARDNOTES="+query);
		Connection con = db.getConnection();
		try {
			if ( con != null ) {
				stm = con.createStatement();
				stm.executeUpdate(query);
				result = true;
			}
		} catch( SQLException ex ) {
			throw ex;
		} finally {
			DBService.close(rs, stm, con);
		}
		return result;
	}
	public String getCardNotes(String className, int Id) throws SQLException {
		String notes="";
		PreparedStatement stm = null;
		ResultSet rs = null;
		String query = "SELECT \"Notes\" FROM \""+className+"\" WHERE \"Id\"='"+Id+"'";
		System.out.println("Query GETCARDNOTES="+query);
		Connection con = db.getConnection();
		try {
			if ( con != null ) {
				stm = con.prepareStatement(query);
				rs = stm.executeQuery();
				if ( rs.next() ) {
					notes = rs.getString(1);
				}
			}
		} catch( SQLException ex ) {
			throw ex;
		} finally {
			DBService.close(rs, stm, con);
		}
		if (notes==null){ // because null value is used when raises a SQL for classes without Notes 
			notes="";
		}
		return notes;
	}
	public int getCardId(String className, Vector columnVector) throws SQLException {
		// cleaning the null parameters
		Enumeration eni = columnVector.elements();
		int i = 0;
		AssetBean ass = null;
		while (eni.hasMoreElements()){
			ass = (AssetBean)eni.nextElement();
			//System.out.println("nome="+ass.getAName()+" tipo="+ass.getAType()+" valore="+ass.getValue());
			if (ass.isToDelete()){
				System.out.println("Removing "+ass.getName());
				columnVector.remove(i);
			} else {
				System.out.println("Not removing "+ass.getName());
			}
			i++;
		}
		// creating query
		int Id=0;
		PreparedStatement stm = null;
		ResultSet rs = null;
		String query = "SELECT \"Id\" FROM \""+className+"\" WHERE ";
		String data = "";
		Enumeration en = columnVector.elements();
		AssetBean ab = null;
		while (en.hasMoreElements()){
			ab = (AssetBean)en.nextElement();
			if (data.equals("")){
				if (ab.getType().equals("numeric") ||ab.getType().equals("float8") || ab.getType().equals("int4") || ab.getType().equals("lookup")){ // without '' 
					data = "\""+ab.getName()+"\"="+ab.getValue()+"";
				} else { // with '' 
					data = "\""+ab.getName()+"\"='"+ab.getValue()+"'";	
				}
			} else {
				if (ab.getType().equals("numeric") ||ab.getType().equals("float8") || ab.getType().equals("int4") || ab.getType().equals("lookup")){ // without '' 
					data+= " AND \""+ab.getName()+"\"="+ab.getValue()+"";
				} else { // with ''
					data+= " AND \""+ab.getName()+"\"='"+ab.getValue()+"'";					
				}
			}
		}
		query+=data;
		System.out.println("Query GETCARDID="+query);
		Connection con = db.getConnection();
		try {
			if ( con != null ) {
				stm = con.prepareStatement(query);
				rs = stm.executeQuery();
				if ( rs.next() ) {
					Id = rs.getInt(1);
				}
			}
		} catch( SQLException ex ) {
			throw ex;
		} finally {
			DBService.close(rs, stm, con);
		}
		return Id;
	}
	
	// *******************************
	// *** SELECT QUERIES ************
	// *******************************
	
	// *********
	// **other**
	// *********
	public  InheritsBean selectSuperClass(String className) throws SQLException {
		// retrieve classId of the superclass
		InheritsBean obj = null;
		String query = "SELECT parentid,parent,parentdescription,childid,child,childdescription FROM cmdbtreecatalog WHERE child='"+className+"'";
		System.out.println("Query SELECTSUPERCLASS="+query);
		ResultSet rs = null;
		PreparedStatement stm = null;
		Connection con = db.getConnection();
		try {
			if ( con != null ) {
				stm = con.prepareStatement(query);
				rs = stm.executeQuery();
				if ( rs.next() ) { 
					obj = setInheritsObject(rs);
				}
			}
		} catch( SQLException ex ) {
			throw ex;
		} finally { // added**
			DBService.close(rs, stm, con);
		}
		return obj;
	}
	public Vector selectClassTree() throws SQLException {
		// retrieve classId of the superclass
		Vector inherits = new Vector();
		InheritsBean obj = null;
		String query = "SELECT parentid,parent,parentdescription,childid,child,childdescription FROM cmdbtreecatalog ";
		System.out.println("Query SELECTCLASSTREE="+query);
		ResultSet rs = null;
		PreparedStatement stm = null;
		Connection con = db.getConnection();
		try {
			if ( con != null ) {
				stm = con.prepareStatement(query);
				rs = stm.executeQuery();
				while ( rs.next() ) { // while ( rs.next() ) {
					obj = setInheritsObject(rs);
					inherits.add(obj);
				}
			}
		} catch( SQLException ex ) {
			throw ex;
		} finally { // added**
			DBService.close(rs, stm, con);
		}
		return inherits;
	}
	// ********
	// **card**
	// ********
	public  Vector selectCardsId(String query) throws SQLException {
		Vector rowIdVector = new Vector();
		// replacing select parameters with "Id"
		String sq = query.substring(query.indexOf("FROM"),query.length());
		query = "SELECT \"Id\",\"IdClass\" "+sq+" ORDER BY \"IdClass\",\"Id\" "; // same ORDER BY of search page
		ResultSet rs = null;
		PreparedStatement stm = null;
		System.out.println("Query SELECTCARDSID="+query);
		Connection con = db.getConnection();
		try {
			if ( con != null ) {
				stm = con.prepareStatement(query);
				rs = stm.executeQuery();
				while ( rs.next() ) {
					rowIdVector.add( new IdBean( rs.getInt(1),rs.getInt(2)) );
				}
			}
		} catch( SQLException ex ) {
			throw ex;
		} finally {
			DBService.close(rs, stm, con);
		}
		return rowIdVector;
	}
	public  int countCards(String query) throws SQLException {
		int count=0;
		// replacing select parameters with "Id"
		String sq = query.substring(query.indexOf("FROM"),query.length());
		query = "SELECT COUNT(*) "+sq+" ";
		ResultSet rs = null;
		PreparedStatement stm = null;
		System.out.println("Query COUNTCARDS="+query);
		Connection con = db.getConnection();
		try {
			if ( con != null ) {
				stm = con.prepareStatement(query);
				rs = stm.executeQuery();
				if ( rs.next() ) {
					count = rs.getInt(1);
				}
			}
		} catch( SQLException ex ) {
			throw ex;
		} finally {
			DBService.close(rs, stm, con);
		}
		return count;
	}
	// used in popups windows / relations searches
	public Vector selectFilteredCardsId(String name, Vector filterVector) throws SQLException {
		Vector rowIdVector = new Vector();
		// creating the query
		String query = "SELECT \"Id\" FROM \""+name+"\" WHERE \"Status\"='A'";
		Enumeration en = filterVector.elements();
		String sq = "";
		String sqq="";
		while (en.hasMoreElements()){
			sqq = (String)en.nextElement();
			sq+=" AND "+sqq;
		}
		query = query + sq;
		System.out.println("Query SELECTFILTEREDCARDID="+query);
		ResultSet rs = null;
		PreparedStatement stm = null;
		Connection con = db.getConnection();
		try {
			if ( con != null ) {
				stm = con.prepareStatement(query);
				rs = stm.executeQuery();
				while ( rs.next() ) {
					rowIdVector.add( new IdBean( rs.getInt(1)) );
				}
			}
		} catch( SQLException ex ) {
			throw ex;
		} finally {
			DBService.close(rs, stm, con);
		}
		return rowIdVector;
	}
	// used in searchForm, reports page
	public Vector selectCards(String query, Vector selectVector, int limit, int offset) throws SQLException {
		Vector reportVector = new Vector();
		String beforeFrom = query.substring(0,query.indexOf("FROM"));
		String from = query.substring(query.indexOf("FROM"),query.indexOf("WHERE"));
		String where = query.substring(query.indexOf("WHERE"),query.length());
		beforeFrom += ", pg_description.description ";
		from += " LEFT JOIN pg_class on pg_class.oid=\"IdClass\" LEFT JOIN pg_description ON pg_description.objoid = pg_class.oid AND pg_description.objsubid = 0 ";
		query = beforeFrom + from + where;
		// adding order
		query += " ORDER BY \"IdClass\",\"Id\" ";
		if (limit!=0){
			query+= " LIMIT "+limit;
		}
		if (offset!=0){
			query+= " OFFSET "+offset;
		}
		System.out.println("Query SELECTCARDS="+query);
		ResultSet rs = null;
		PreparedStatement stm = null;
		Vector assetVector = null;
		Connection con = db.getConnection();
		try {
			if ( con != null ) {
				stm = con.prepareStatement(query);
				rs = stm.executeQuery();
				while ( rs.next() ) {
					assetVector = setReportObject(rs,selectVector);
					//System.out.println("reportVector: adding a new vetor of size "+assetVector.size());
					reportVector.add(assetVector);
				}
			}
		} catch( SQLException ex ) {
			throw ex;
		} finally {
			DBService.close(rs, stm, con);
		}
		return reportVector;
	}
	public  Vector selectCard(String name, int rowId, Vector attributeBeanVector) throws SQLException {
		Vector columnVector = new Vector();
		// select attname,typname from pg_attribute,pg_type where attrelid=46701 AND attstattarget=-1 AND atttypid=pg_type.oid
		// makes the query ad hoc
		String query = "";
		Enumeration e = attributeBeanVector.elements();
		AttributeBean bean = new AttributeBean();
		while (e.hasMoreElements()){
			bean = (AttributeBean)e.nextElement();
			if (query.equals("")){
				query+="SELECT \""+name+"\".\""+bean.getName()+"\"";
			} else {
				query+=",\""+name+"\".\""+bean.getName()+"\"";
			}
		}
		query+= " FROM \""+name+"\" WHERE \""+name+"\".\"Id\"="+rowId;
		System.out.println("Query SELECTCARD="+query);
		AssetBean ass = null;
		AttributeBean att = null;
		ResultSet rs = null;
		PreparedStatement stm = null;
		Connection con = db.getConnection();
		try {
			if ( con != null ) {
				stm = con.prepareStatement(query);
				rs = stm.executeQuery();
				if ( rs.next() ) { // while ( rs.next() ) {
					int i = 1;
					Enumeration eni = attributeBeanVector.elements();
					while (eni.hasMoreElements()){
						att = (AttributeBean)eni.nextElement();
						ass = new AssetBean(att);
						ass.setValue(rs.getString(i));
						columnVector.add(ass);
						i++;
					}
				} else { // This is the card without data, necessary to construct the pages
					int i = 1;
					Enumeration eni = attributeBeanVector.elements();
					while (eni.hasMoreElements()){
						att = (AttributeBean)eni.nextElement();
						ass = new AssetBean(att);
						columnVector.add(ass);
						i++;
					}
				}
			}
		} catch( SQLException ex ) {
			throw ex;
		} finally {
			DBService.close(rs, stm, con);
		}
		return columnVector;
	}
	// used only in HistoryPopupForm
	public  Vector selectCardHistory(String name, int rowId, Vector attributeBeanVector) throws SQLException {
		if (name.indexOf("_history")==-1){
			name = name+"_history";
		}
		Vector columnVector = new Vector();
		// don't show Notes
		Enumeration enim = attributeBeanVector.elements();
		AttributeBean ab = null;
		int ir = 0;
		boolean notesFound = false;
		while (enim.hasMoreElements()){
			ab = (AttributeBean)enim.nextElement();
			if (ab.getName().equals("Notes")){
				notesFound = true;
				break;
			}
			ir++;
		}
		if (notesFound){
			attributeBeanVector.removeElementAt(ir);
		}
		String query = "";
		Enumeration e = attributeBeanVector.elements();
		AttributeBean bean = new AttributeBean();
		while (e.hasMoreElements()){
			bean = (AttributeBean)e.nextElement();
			if (query.equals("")){
				query+="SELECT \""+name+"\".\""+bean.getName()+"\"";
			} else {
				query+=",\""+name+"\".\""+bean.getName()+"\"";
			}
		}
		query+= " FROM \""+name+"\" WHERE \""+name+"\".\"CurrentId\"="+rowId;  // old was Id
		System.out.println("Query SELECTCARD="+query);
		AssetBean ass = null;
		AttributeBean att = null;
		ResultSet rs = null;
		PreparedStatement stm = null;
		Connection con = db.getConnection();
		try {
			if ( con != null ) {
				stm = con.prepareStatement(query);
				rs = stm.executeQuery();
				if ( rs.next() ) { // while ( rs.next() ) {
					int i = 1;
					Enumeration eni = attributeBeanVector.elements();
					while (eni.hasMoreElements()){
						att = (AttributeBean)eni.nextElement();
						ass = new AssetBean(att);
						ass.setValue(rs.getString(i));
						columnVector.add(ass);
						i++;
					}
				} else { // This is the card without data, necessary to construct the pages
					int i = 1;
					Enumeration eni = attributeBeanVector.elements();
					while (eni.hasMoreElements()){
						att = (AttributeBean)eni.nextElement();
						ass = new AssetBean(att);
						columnVector.add(ass);
						i++;
					}
				}
			}
		} catch( SQLException ex ) {
			throw ex;
		} finally {
			DBService.close(rs, stm, con);
		}
		return columnVector;
	}
	public  LabelsBean selectCardLabels(String name, int rowId) throws SQLException {
		LabelsBean bean = new LabelsBean();
		String query = "SELECT  ";
		ResultSet rs = null;
		PreparedStatement stm = null;
		query = "SELECT \"Code\",\"Description\" FROM \""+name+"\" WHERE \"Id\"="+rowId;
		System.out.println("Query SELECTCARDLABELS="+query);
		Connection con = db.getConnection();
		try {
			if ( con != null ) {
				stm = con.prepareStatement(query);
				rs = stm.executeQuery();
				if ( rs.next() ) { // while ( rs.next() ) {
					bean.setClassName(name);
					bean.setCode(rs.getString(1));
					bean.setDescription(rs.getString(2));
				}
			}
		} catch( SQLException ex ) {
			throw ex;
		} finally {
			DBService.close(rs, stm, con);
		}
		return bean;
	}

	// ***********
	// **history**
	// ***********
	public Vector selectHistory(String className, int classId, int objId) throws SQLException {
		String query1 = "SELECT relname FROM pg_class WHERE pg_class.oid="+classId+" ";
		System.out.println("Query1 SELECTHISTORY="+query1);
		ResultSet rs1 = null;
		PreparedStatement pstm1 = null;
		Connection con = db.getConnection();
		try {
			if ( con != null ) {
				pstm1 = con.prepareStatement(query1);
				rs1 = pstm1.executeQuery();
				if ( rs1.next() ) { // while ( rs.next() ) {
					className = rs1.getString(1);
				}
			}
		} catch( SQLException ex ) {
			throw ex;
		} finally { // added**
			DBService.close(rs1, pstm1, null);
		}
		if (className.indexOf("_history")==-1){
			className = className+"_history";
		}
		Vector historyVector = new Vector(); 
		//String query = "select \"Id\",\"IdClass\",\"User\",\"BeginDate\",\"CurrentId\",\"EndDate\" from \""+className+"\" WHERE \"IdClass\"='"+classId+"' AND \"CurrentId\"='"+objId+"'";
		String query2 = "SELECT \"User\",\"BeginDate\",\"CurrentId\",\"EndDate\" from \""+className+"\" WHERE \"IdClass\"='"+classId+"' AND \"CurrentId\"='"+objId+"'";     
		System.out.println("Query2 SELECTHISTORY="+query2);
		ResultSet rs2 = null;
		PreparedStatement stm2 = null;
		HistoryBean his = null;
		try {
			if ( con != null ) {
				stm2 = con.prepareStatement(query2);
				rs2 = stm2.executeQuery();
				while (rs2.next()) { // while ( rs.next() ) {
					 his = setHistoryObject(rs2);
					 his.setClassId(classId);
					 his.setObjId(objId);
					 his.setAttribute(true);
					 historyVector.add(his);
				}
			}
		} catch( SQLException ex ) {
			throw ex;
		} finally {
			DBService.close(rs2, stm2, con);
		}
		return historyVector;
	}


	/**************************/
	/* USEFUL SETTING METHOTS */
	/**************************/
	
	private ClassBean setClassObject(ResultSet rs) {
		ClassBean classBean = new ClassBean();
		try {
			classBean.setId(rs.getInt(1));
			classBean.setName(rs.getString(2));
			classBean.setComment(rs.getString(3));
			classBean.setIdCategory(rs.getInt(4));
			classBean.setCategory(rs.getString(5));
			classBean.setDescription(rs.getString(6));
			classBean.setMode(rs.getString(7));
			String sup = rs.getString(8);
			//System.out.println("setClassObject sup="+sup);
			if (sup.equals("true")){
				classBean.setSuperClass(true);
			} else {
				classBean.setSuperClass(false);
			}
			classBean.setManager(rs.getString(9));
			classBean.setStatus(rs.getString(10));
			//System.out.println("setClassObject cat="+cat);
			/*
			if (cat!=null && cat.length()>50) {
				cat = cat.substring(0,50);
				cat = cat+"...";
			}
			*/
		} catch (SQLException e) {
			e.printStackTrace();
		}
		return classBean;
	}
	
	private AttributeBean setAttributeObject(ResultSet rs) {
		AttributeBean bean = new AttributeBean();		
		try {
			// class info
			bean.getClassBean().setId(rs.getInt(1));
			bean.getClassBean().setName(rs.getString(2));
			bean.getClassBean().setComment(rs.getString(3));
			bean.getClassBean().setIdCategory(rs.getInt(4));
			bean.getClassBean().setCategory(rs.getString(5));
			bean.getClassBean().setDescription(rs.getString(6));
			bean.getClassBean().setMode(rs.getString(7));
			String sup = rs.getString(8);
			if (sup.equals("true")){
				bean.getClassBean().setSuperClass(true);
			} else {
				bean.getClassBean().setSuperClass(false);
			}
			bean.getClassBean().setManager(rs.getString(9));
			bean.getClassBean().setStatus(rs.getString(10));
			// attribute info
			bean.setName(rs.getString(11));
			bean.setSQLType(rs.getString(12));
			bean.setType(rs.getString(12)); // transform SQLType in aType, same getString
			bean.setLength(rs.getInt(13)); // attributelength (String length)
			bean.setMandatory(rs.getString(14));
			bean.setDefaultValue(rs.getString(15)); // default
			bean.setMode(rs.getString(16)); // mode
			bean.setDescription(rs.getString(17));
			bean.setLookup(rs.getString(18));
			bean.setBaseDSP(rs.getString(19)); // baseDSP
			bean.setStatus(rs.getString(20));  // status
			bean.setIndex(rs.getInt(21)); // logical-index
			bean.setDBIndex(rs.getInt(22)); // db-index
			bean.setComment(rs.getString(23)); // attribute comment
		} catch (SQLException e) {
			e.printStackTrace();
		}
		return bean;
	}
	
	private Vector setReportObject(ResultSet rs, Vector selectVector) {
		Vector assetVector = new Vector();
		AssetBean as = null;
		AttributeBean ab = null;
		String comment ="";
		try {
			Enumeration en = selectVector.elements();
			int i = 1;
			while (en.hasMoreElements()){
				ab = (AttributeBean)en.nextElement();
				as = new AssetBean(ab);
				as.setValue(rs.getString(i));
				// adding class description
				comment = rs.getString(selectVector.size()+1); // this is the position of pg_description.description
				if (comment!=null && comment.indexOf("DESCR:")!=-1){
					comment = comment.substring(comment.indexOf("DESCR:")+7,comment.length());
					if (comment.indexOf("|")!=-1){
						comment = comment.substring(0,comment.indexOf("|"));
					}
					as.getClassBean().setDescription(comment);
				} else {
					as.getClassBean().setDescription("");
				}
				assetVector.add(as);
				i++;
			}
			// if the search name was Asset the overwrites every IdClass
			int IdClass = 0;
			Enumeration en2 = assetVector.elements();
			while (en2.hasMoreElements()){
				as = (AssetBean)en2.nextElement();
				if (as.getName().equals("IdClass")){
					IdClass = Integer.parseInt(as.getValue());
					break;
				}
			}
			Enumeration en3 = assetVector.elements();
			while (en3.hasMoreElements()){
				as = (AssetBean)en3.nextElement();
				as.getClassBean().setId(IdClass);
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}		
		return assetVector;
	}
	
	private HistoryBean setHistoryObject(ResultSet rs) {
		HistoryBean bean = new HistoryBean();		
		try {
			bean.setUser(rs.getString(1));
			bean.setBeginDate(rs.getString(2));
			bean.setCurrentId(rs.getInt(3));
			bean.setEndDate(rs.getString(4));
		} catch (SQLException e) {
			e.printStackTrace();
		}		
		return bean;
	}

	private InheritsBean setInheritsObject(ResultSet rs) {
		InheritsBean bean = new InheritsBean();		
		try {
			bean.setParentId(rs.getInt(1));
			bean.setParent(rs.getString(2));
			bean.setParentDescription(rs.getString(3));
			bean.setChildId(rs.getInt(4));
			bean.setChild(rs.getString(5));
			bean.setChildDescription(rs.getString(6));
		} catch (SQLException e) {
			e.printStackTrace();
		}		
		return bean;
	}
	
	/***********************/
	/* GETTERS AND SETTERS */
	/***********************/

	public Hashtable getAttributeHash() {
		return attributeHash;
	}
	public void setAttributeHash(Hashtable attributeHash) {
		this.attributeHash = attributeHash;
	}
	public Vector getAttributeVector() {
		return attributeVector;
	}
	public void setAttributeVector(Vector attributeVector) {
		this.attributeVector = attributeVector;
	}
	public ClassBean getBean() {
		return bean;
	}
	public void setBean(ClassBean bean) {
		this.bean = bean;
	}
	public Presentation getPresentation() {
		Presentation pres;
		pres = new HTMLclass();
		System.out.println("Dentro getPresentation");
		if (presentation==null){
			//presentation = (Presentation)load(pres,classManager);
			presentation = load(pres,classManager);
		}
		return presentation;
	}
	public void setPresentation(Presentation presentation) {
		this.presentation = presentation;
	}
	public String getUser() {
		return user;
	}
	public void setUser(String user) {
		this.user = user;
	}
	public String getClassManager() {
		return classManager;
	}
	public void setClassManager(String classManager) {
		this.classManager = classManager;
	}
	public DBService getDb() {
		return db;
	}
	public void setDb(DBService db) {
		this.db = db;
	}
	

} // end of class
