package cmdbuild.utils;

import java.util.ArrayList;

public class CSVReaderExample {

	private String strData		= "";
	private char chSeparator	= ',';
	private boolean bEscapes	= false;

	public CSVReaderExample() {
		super();
	}


	public char guessSeparator(){
		char[] arSeparators = new char[]{ ',', ';', '	', ':'};
		int[] arPoints		= new int[arSeparators.length];
		for (int nA = 0; nA < arSeparators.length; nA++) {
			arPoints[nA] = 0;
			//number of >",<
			arPoints[nA] += countOccurences( this.strData, "\"" + arSeparators[nA]) * 5;
			//number of >,"<
			arPoints[nA] += countOccurences( this.strData, arSeparators[nA] + "\"") * 5;
			//number of >,<
			arPoints[nA] += countOccurences( this.strData, arSeparators[nA] + "");
		}

		//find the one with the maximum points
		int nBestIndex		= 0;
		for( int nA = 1; nA < arPoints.length; nA++ ) {
			if (arPoints[nA] > arPoints[nBestIndex]) {
				nBestIndex = nA;
			}
		}

		return arSeparators[nBestIndex];
	}//public char guessSeparator()

	public String[][] parseData(){
		return this.parseData( 0);
	}

	public String[][] parseData(int nLines)	{
		int nDataLength = this.strData.length();
		int nPos		= 0;
		ArrayList arList;
		if (nLines == 0) {
			//FIXME: if there are only \r for newlines (mac) then this doesn't work
				//arList = new ArrayList(StringHelper.countOccurences( this.strData, "\n"));
			arList = new ArrayList();
				nLines = -1;
			} else {
				arList = new ArrayList(nLines);
			}
			int nCurrentLine	= 0;

			char chCur	= ' ';
			char chNext	= ' ';


			//if we're currently in quotes
			boolean bInQuotes		= false;
			boolean bInField		= true;

			//the current field value
			StringBuffer strCurrent	= new StringBuffer(10);

			while (nPos < nDataLength) {
				chCur = this.getChar( nPos);

				if (arList.size() <= nCurrentLine || arList.get( nCurrentLine) == null) {
					nLines--;
					if (nLines == -1) {
						//user doesn't want more lines
						break;
					}
					arList.add( nCurrentLine, new ArrayList());
				}
				if (bInField) {
					if (chCur == '"') {
						//a quote...
						if (!bInQuotes) {
							//now we're in the quote
							//try to remove the stuff before if just spaces
							if (strCurrent.toString().trim().length() == 0) {
								strCurrent = new StringBuffer();
							}
							bInQuotes = true;
						} else if (this.getChar( nPos + 1) == '"') {
							//double quote!
							nPos++;//skip the second quote
							strCurrent.append( '"');
						} else {
							//seems as we're finished, closing quote
							bInQuotes	= false;
							((ArrayList)arList.get( nCurrentLine)).add( strCurrent.toString());
							strCurrent = new StringBuffer(10);
							bInField = false;
						}
					} else {
						//this is not a quote character

						if (chCur == this.chSeparator && !bInQuotes) {
							//we've got a separator and we're not in quotes
							//so this field is done
							((ArrayList)arList.get( nCurrentLine)).add( strCurrent.toString().trim());
							strCurrent = new StringBuffer(10);
						} else if (!bInQuotes && (chCur == '\r' || chCur == '\n')) {
							//not in quotes and CR or LF -> end of field AND end of line
							((ArrayList)arList.get( nCurrentLine)).add( strCurrent.toString().trim());
							strCurrent = new StringBuffer(10);
							nCurrentLine++;

							if (chCur == '\r' && this.getChar( nPos + 1) == '\n') {
								//skip the \n after the \r
								nPos++;
							}
						} else {
							if (this.bEscapes && chCur == '\\') {
								//escape char. what to do now?
								chNext	= this.getChar( nPos + 1);
								if (chNext == '"') {
									//escaped quote
									strCurrent.append( "\"");
									nPos++;
								} else if (chNext == 'r') {
									//escaped CR
									strCurrent.append( "\r");
									nPos++;
								} else if (chNext == 'n') {
									//escaped LF
									strCurrent.append( "\n");
									nPos++;
								} else if (chNext == '\\') {
									//escaped escape char (backslash)
									strCurrent.append( "\\");
									nPos++;
								} else {
									//just append the backslash
									strCurrent.append( chCur);
								}
							} else {
								strCurrent.append( chCur);
							}
						}
					}//not a quote char
				} else {
					//we're outside a field until the separator comes again
					if (chCur == this.chSeparator) {
						bInField	= true;
					}
				}

				nPos++;
			}//while nPos < nDataLength

			//if length > 0 OR the last char in the file was the separator -> add the field
			if (strCurrent.length() > 0 || this.getChar( nPos - 1) == this.chSeparator) {
				((ArrayList)arList.get( nCurrentLine)).add( strCurrent.toString().trim());
			}

//			System.out.println( "::" + arList + "::");

			int nFoundLines		= arList.size();
			if (nFoundLines == 0) {
				return new String[0][0];
			}

			int nFoundFields	= ((ArrayList)arList.get(0)).size();
			String[][] arData = new String[nFoundLines][nFoundFields];
			for (int nA = 0; nA < nFoundLines; nA++) {
				for (int nB = 0; nB < nFoundFields; nB++) {
					try {
						arData[nA][nB] = (String)((ArrayList)arList.get(nA)).get(nB);
					} catch (IndexOutOfBoundsException e) {}
				}
			}

			return arData;
		}//public String[][] parseData(int nLines)



		public char getChar(int nPos){
			char[] dst = new char[1];
			try {
				this.strData.getChars( nPos, nPos + 1, dst, 0);
			} catch (StringIndexOutOfBoundsException e) {
				//null character
				return '\0';
			}
			return dst[0];
		}//public char getChar(int nPos)



		/**
		 * counts the number of occurences of strSearch in the strData string
		 *
		 * @param	strData		The string which shall be searched
		 * @param	strSearch	The search string
		 */
		public static int countOccurences(String strData, String strSearch){
			if (strSearch.length() == 0) {
				return 0;
			}

			int nCount = 0;
			int nPos = -1;
			do {
				nPos = strData.indexOf( strSearch, nPos);
				if (nPos == -1) {
					break;
				}
				nCount++;
				nPos = nPos + strSearch.length();
			} while (nPos < strData.length());

			return nCount;
		}
}  // end of class
